package betterwithmods.common.entity;

import betterwithmods.BWMod;
import betterwithmods.common.BWMBlocks;
import betterwithmods.common.blocks.mechanical.tile.TileEntityPulley;
import betterwithmods.module.GlobalConfig;
import betterwithmods.util.AABBArray;
import betterwithmods.util.InvUtils;
import com.google.common.collect.Maps;
import com.google.common.collect.Sets;
import io.netty.buffer.ByteBuf;
import io.netty.buffer.Unpooled;
import net.minecraft.block.Block;
import net.minecraft.block.BlockRailBase;
import net.minecraft.block.BlockRedstoneWire;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.Entity;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Blocks;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTTagList;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.ResourceLocation;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.Vec3i;
import net.minecraft.world.World;
import net.minecraftforge.event.world.GetCollisionBoxesEvent;
import net.minecraftforge.fml.common.Mod;
import net.minecraftforge.fml.common.eventhandler.SubscribeEvent;
import net.minecraftforge.fml.common.registry.IEntityAdditionalSpawnData;

import java.nio.charset.Charset;
import java.util.*;
import java.util.Map.Entry;

@Mod.EventBusSubscriber(modid = BWMod.MODID)
public class EntityExtendingRope extends Entity implements IEntityAdditionalSpawnData {

    public AABBArray blockBB;
    private BlockPos pulley;
    private int targetY;
    private boolean up;
    private Map<Vec3i, IBlockState> blocks;
    private Map<Vec3i, NBTTagCompound> tiles;
    private float speed = 0.12f;

    public EntityExtendingRope(World worldIn) {
        this(worldIn, null, null, 0);
    }

    public EntityExtendingRope(World worldIn, BlockPos pulley, BlockPos source, int targetY) {
        super(worldIn);
        this.pulley = pulley;
        this.targetY = targetY;
        if (source != null) {
            this.up = source.func_177956_o() < targetY;
            this.func_70107_b(source.func_177958_n() + 0.5, source.func_177956_o(), source.func_177952_p() + 0.5);
        }
        this.blocks = Maps.newHashMap();
        this.tiles = Maps.newHashMap();
        this.blockBB = null;
        this.func_70105_a(0.1F, 1F);
        this.field_70158_ak = true;
    }

    @Override
    public void func_70107_b(double x, double y, double z) {
        double pY = field_70163_u;

        super.func_70107_b(x, y, z);

        if (blocks != null) updatePassengers(pY, field_70163_u, false);
    }

    @Override
    protected void func_70088_a() {

    }

    @Override
    public float func_70047_e() {
        return -1;
    }

    @Override
    protected void func_70037_a(NBTTagCompound compound) {
        pulley = new BlockPos(compound.func_74762_e("PulleyX"), compound.func_74762_e("PulleyY"),
                compound.func_74762_e("PulleyZ"));
        targetY = compound.func_74762_e("TargetY");
        up = compound.func_74767_n("Up");
        if (compound.func_74764_b("BlockData")) {
            byte[] bytes = compound.func_74770_j("BlockData");
            ByteBuf buf = Unpooled.buffer(bytes.length);
            buf.writeBytes(bytes);
            blocks = deserializeBlockmap(buf);
        }
        if (compound.func_74764_b("Tiles")) {
            tiles = deserializeTiles(compound.func_74775_l("Tiles"));
        }
        rebuildBlockBoundingBox();
    }

    @Override
    protected void func_70014_b(NBTTagCompound compound) {
        compound.func_74768_a("PulleyX", pulley.func_177958_n());
        compound.func_74768_a("PulleyY", pulley.func_177956_o());
        compound.func_74768_a("PulleyZ", pulley.func_177952_p());
        compound.func_74768_a("TargetY", targetY);
        compound.func_74757_a("Up", up);
        ByteBuf buf = Unpooled.buffer();
        serializeBlockmap(buf, blocks);
        byte[] bytes = new byte[buf.readableBytes()];
        buf.readBytes(bytes);
        compound.func_74773_a("BlockData", bytes);
        NBTTagCompound t = new NBTTagCompound();
        serializeTiles(t, tiles);
        compound.func_74782_a("Tiles", t);
        if (GlobalConfig.debug) {
            for (int i = 0; i < bytes.length; i++) {
                if (i % 16 == 0) {
                    StringBuilder text = new StringBuilder(Integer.toHexString(i));
                    while (text.length() < 8) {
                        text.insert(0, "0");
                    }
                    System.out.print("\n" + text + ": ");
                }
                StringBuilder b = new StringBuilder(Integer.toHexString(Byte.toUnsignedInt(bytes[i])));
                while (b.length() < 2) {
                    b.insert(0, "0");
                }
                System.out.print(b);
                if (i % 2 == 1) {
                    System.out.print(' ');
                }
            }
        }
    }

    private void serializeTiles(NBTTagCompound tag, Map<Vec3i, NBTTagCompound> tiles) {
        NBTTagList list = new NBTTagList();
        tiles.forEach((vec, tile) -> {
            NBTTagCompound entry = new NBTTagCompound();
            entry.func_74772_a("offset", new BlockPos(vec).func_177986_g());
            entry.func_74782_a("tile", tile);
            list.func_74742_a(entry);
        });
        tag.func_74782_a("entries", list);
    }

    private Map<Vec3i, NBTTagCompound> deserializeTiles(NBTTagCompound tag) {
        Map<Vec3i, NBTTagCompound> map = new HashMap<>();
        NBTTagList list = tag.func_150295_c("entries", 10);
        list.iterator().forEachRemaining(e -> {
            NBTTagCompound entry = (NBTTagCompound) e;
            Vec3i offset = BlockPos.func_177969_a(entry.func_74763_f("offset"));
            NBTTagCompound tileData = entry.func_74775_l("tile");
            map.put(offset, tileData);
        });
        return map;
    }

    private void serializeBlockmap(ByteBuf buf, Map<Vec3i, IBlockState> blocks) {
        buf.writeInt(blocks.size());
        IBlockState state;
        for (Vec3i vec : blocks.keySet()) {
            state = blocks.get(vec);
            if (state != null) {
                buf.writeInt(vec.func_177958_n());
                buf.writeInt(vec.func_177956_o());
                buf.writeInt(vec.func_177952_p());

                Block block = state.func_177230_c();
                ResourceLocation resourcelocation = state.func_177230_c().getRegistryName();
                if (resourcelocation != null) {
                    String blockName = resourcelocation.toString();
                    buf.writeInt(blockName.length());
                    buf.writeBytes(blockName.getBytes(Charset.forName("UTF-8")));
                    buf.writeByte((byte) block.func_176201_c(state));
                }
            }
        }

    }

    private Map<Vec3i, IBlockState> deserializeBlockmap(ByteBuf buf) {
        Map<Vec3i, IBlockState> map = new HashMap<>();
        int size = buf.readInt();
        for (int i = 0; i < size; i++) {
            Vec3i vec = new Vec3i(buf.readInt(), buf.readInt(), buf.readInt());
            int len = buf.readInt();
            byte[] bytes = new byte[len];
            buf.readBytes(bytes);
            String name = new String(bytes, Charset.forName("UTF-8"));
            int meta = buf.readByte();
            //TODO this cannot stay.
            @SuppressWarnings("deprecation")
            IBlockState state = Block.func_149684_b(name).func_176203_a(meta);
            map.put(vec, state);
        }
        return map;
    }

    private void rebuildBlockBoundingBox() {
        if (blocks == null || blocks.isEmpty()) {
            this.blockBB = null;
        } else {
            List<AxisAlignedBB> bbs = new ArrayList<>();
            bbs.add(new AxisAlignedBB(0.45, 0, 0.45, 0.55, 1, 0.55)); // rope bounding box
            for (Vec3i vec : blocks.keySet()) {
                IBlockState state = blocks.get(vec);
                if (state.func_177230_c().func_149703_v()) {
                    AxisAlignedBB bb2 = new AxisAlignedBB(vec.func_177958_n(), vec.func_177956_o(), vec.func_177952_p(), vec.func_177958_n() + 1,
                            vec.func_177956_o() + getBlockStateHeight(state), vec.func_177952_p() + 1);
                    bbs.add(bb2);
                }
            }
            this.blockBB = new AABBArray(bbs.toArray(new AxisAlignedBB[0])).func_72317_d(-0.5, 0, -0.5);
        }
    }

    private double getSpeed() {
        return up ? speed : -speed;
    }

    @Override
    public void func_70071_h_() {
        if (up) {
            if (field_70163_u > targetY) {
                if (done())
                    return;
            }
        } else {
            if (field_70163_u < targetY) {
                if (done())
                    return;
            }
        }

        this.field_70169_q = this.field_70165_t;
        this.field_70167_r = this.field_70163_u;
        this.field_70166_s = this.field_70161_v;

        func_70107_b(
                pulley.func_177958_n() + 0.5,
                this.field_70163_u + getSpeed(),
                pulley.func_177952_p() + 0.5
        );

        this.field_70170_p.func_72866_a(this, false);
    }

    public void updatePassengers(double posY, double newPosY, boolean b) {
        if (blockBB == null || field_70170_p == null) return;
        AABBArray oldBB = blockBB.func_72317_d(field_70165_t, posY, field_70161_v);
        AABBArray newBB = blockBB.func_72317_d(field_70165_t, newPosY, field_70161_v);
        Set<Entity> passengers = Sets.newHashSet(func_130014_f_().func_175647_a(Entity.class, newBB, e -> !(e instanceof EntityExtendingRope)));
        for (Entity e : passengers) {
            AxisAlignedBB ebb = e.func_174813_aQ();
            if (!newBB.func_72326_a(ebb)) continue;

            double yoff = -oldBB.func_72323_b(ebb, posY - newPosY);

            if (yoff != 0) {
                if (func_130014_f_().field_72995_K || !(e instanceof EntityPlayer) || b)
                    e.func_70091_d(null, 0, yoff, 0);

                e.field_70181_x = up ? 0 : -speed;
                e.field_70160_al = false;
                e.field_70122_E = true;
                e.field_70132_H = e.field_70124_G = true;
                e.field_70143_R = 0;
            }
        }
    }

    private double getBlockStateHeight(IBlockState blockState) {
        return (blockState == null ? 1
                : (blockState.func_177230_c() == BWMBlocks.ANCHOR ? 0.375F
                : (blockState.func_177230_c() instanceof BlockRailBase || blockState.func_177230_c() instanceof BlockRedstoneWire ? 0 : 1)));
    }

    @Override
    public boolean func_90999_ad() {
        return false;
    }

    private void reconstruct() {
        BlockPos pos = this.pulley.func_177979_c(this.pulley.func_177956_o() - targetY);

        int retries = 0;
        while (!blocks.isEmpty() && retries < 10) {
            retries++;
            int skipped = 0;
            for (Entry<Vec3i, IBlockState> entry : blocks.entrySet()) {
                BlockPos blockPos = pos.func_177971_a(entry.getKey());
                IBlockState state = entry.getValue();
                if (state.func_177230_c().func_176196_c(func_130014_f_(), blockPos)) {

                    func_130014_f_().func_180501_a(blockPos, state, 3);
                    if (tiles.containsKey(entry.getKey())) {
                        TileEntity tile = func_130014_f_().func_175625_s(blockPos);
                        if (tile != null) {
                            NBTTagCompound tag = tiles.get(entry.getKey());
                            tile.func_145839_a(tag);
                            tile.func_174878_a(blockPos);
                        }
                    }
                    blocks.remove(entry.getKey());
                    tiles.remove(entry.getKey());
                    skipped = 0;
                    break;
                }
                skipped++;
            }
            if (skipped == 0) {
                retries = 0;
            }
        }

        if (retries > 0) {
            blocks.forEach((vec, state) -> state.func_177230_c().getDrops(func_130014_f_(), pos, state, 0).forEach(stack -> InvUtils.spawnStack(func_130014_f_(), field_70165_t, field_70163_u, field_70161_v, stack, 10)));
        }

        updatePassengers(field_70163_u, field_70163_u + (up ? 0 : 0.5), true);
    }

    private boolean done() {
        if (!func_130014_f_().field_72995_K) {
            TileEntity te = func_130014_f_().func_175625_s(pulley);
            if (te instanceof TileEntityPulley) {
                TileEntityPulley pulley = (TileEntityPulley) te;
                if (!pulley.onJobCompleted(up, targetY, this)) {
                    reconstruct();
                    return true;
                }
            } else {
                //The tile has been lost, abort
                reconstruct();
                this.func_70106_y();
                return true;
            }
        }
        return false;
    }

    public void addBlock(Vec3i offset, World world, BlockPos pos) {
        IBlockState state = world.func_180495_p(pos).func_185899_b(world, pos);
        TileEntity tile = world.func_175625_s(pos);
        blocks.put(offset, state);
        if (tile != null) {
            NBTTagCompound tag = new NBTTagCompound();
            tile.func_189515_b(tag);
            tiles.put(offset, tag);
            world.func_175713_t(pos);
        }
        rebuildBlockBoundingBox();
    }

    @Override
    public void writeSpawnData(ByteBuf buffer) {
        buffer.writeInt(pulley.func_177958_n());
        buffer.writeInt(pulley.func_177956_o());
        buffer.writeInt(pulley.func_177952_p());
        buffer.writeInt(targetY);
        buffer.writeBoolean(up);
        serializeBlockmap(buffer, blocks);
    }

    @Override
    public void readSpawnData(ByteBuf additionalData) {
        pulley = new BlockPos(additionalData.readInt(), additionalData.readInt(), additionalData.readInt());
        targetY = additionalData.readInt();
        up = additionalData.readBoolean();
        blocks = deserializeBlockmap(additionalData);
    }

    public void setTargetY(int i) {
        this.targetY = i;
    }

    public boolean getUp() {
        return up;
    }

    @Override
    protected boolean func_184219_q(Entity passenger) {
        return false;
    }

    public boolean isPathBlocked() {
        HashSet<BlockPos> blocked = new HashSet<>();
        blocks.forEach((vec, state) -> {
            if (blocked.isEmpty() && !up || state.func_177230_c() != BWMBlocks.ANCHOR) {
                BlockPos pos = this.pulley.func_177979_c(this.pulley.func_177956_o() - targetY).func_177971_a(vec);
                if (up)
                    pos = pos.func_177984_a();
                else
                    pos = pos.func_177977_b();

                Block b = func_130014_f_().func_180495_p(pos).func_177230_c();

                if (!(b == Blocks.field_150350_a || b.func_176200_f(func_130014_f_(), pos))) {
                    blocked.add(pos);
                }
            }
        });
        return !blocked.isEmpty();
    }

    @Override
    public boolean func_70039_c(NBTTagCompound compound) {
        return false;
    }

    public BlockPos getPulleyPosition() {
        return this.pulley;
    }

    public Map<Vec3i, IBlockState> getBlocks() {
        return blocks;
    }

    public Map<Vec3i, NBTTagCompound> getTiles() {
        return tiles;
    }

    @Override
    protected void func_70105_a(float width, float height) {
        if (blockBB == null)
            super.func_70105_a(width, height);
    }

    @Override
    public void func_174826_a(AxisAlignedBB bb) {
        rebuildBlockBoundingBox();
        super.func_174826_a(blockBB != null ? blockBB.func_72317_d(this.field_70165_t, this.field_70163_u, this.field_70161_v) : bb);
    }

    @Override
    public AxisAlignedBB func_70046_E() {
        return (this.func_174813_aQ() instanceof AABBArray
                ? ((AABBArray) this.func_174813_aQ()).forEach(i -> i.func_186666_e(i.field_72337_e - 0.125))
                : this.func_174813_aQ());
    }

    // Handle collision for players only on the client side
    @SubscribeEvent
    public static void getCollisionBoxes(GetCollisionBoxesEvent e) {
        if (e.getEntity() instanceof EntityPlayer && !e.getEntity().field_70170_p.field_72995_K) {
//            e.getCollisionBoxesList().removeIf(it -> it instanceof AABBArray);
        }
    }

}
