package betterwithmods.common.entity.ai;

import net.minecraft.advancements.CriteriaTriggers;
import net.minecraft.entity.EntityAgeable;
import net.minecraft.entity.ai.EntityAIBase;
import net.minecraft.entity.item.EntityXPOrb;
import net.minecraft.entity.passive.EntityAnimal;
import net.minecraft.entity.player.EntityPlayerMP;
import net.minecraft.stats.StatList;
import net.minecraft.util.EnumParticleTypes;
import net.minecraft.util.math.Vec3d;
import net.minecraft.world.World;

import java.util.List;
import java.util.Random;

public class EntityAIMate extends EntityAIBase {


    private final EntityAnimal animal;
    private final Class<? extends EntityAnimal> mateClass;
    private final World world;
    private final double moveSpeed;
    private int spawnBabyDelay;
    private double distance;

    private EntityAnimal targetMate;

    public EntityAIMate(EntityAnimal animal, double speedIn, double distance) {
        this(animal, speedIn, animal.getClass(), distance);
    }

    public EntityAIMate(EntityAnimal p_i47306_1_, double p_i47306_2_, Class<? extends EntityAnimal> p_i47306_4_, double distance) {
        this.animal = p_i47306_1_;
        this.world = p_i47306_1_.field_70170_p;
        this.mateClass = p_i47306_4_;
        this.moveSpeed = p_i47306_2_;
        this.distance = distance;
        this.func_75248_a(3);
    }

    private static Vec3d getMidpoint(Vec3d a, Vec3d b) {
        return new Vec3d((a.field_72450_a + b.field_72450_a) / 2, (a.field_72448_b + b.field_72448_b) / 2, (a.field_72449_c + b.field_72449_c) / 2);
    }

    /**
     * Returns whether the EntityAIBase should begin execution.
     */
    public boolean func_75250_a() {
        if (!this.animal.func_70880_s()) {
            return false;
        } else {
            this.targetMate = this.getNearbyMate();
            return this.targetMate != null;
        }
    }

    /**
     * Returns whether an in-progress EntityAIBase should continue executing
     */
    public boolean func_75253_b() {
        return this.targetMate.func_70089_S() && this.targetMate.func_70880_s() && this.spawnBabyDelay < 60;
    }

    /**
     * Reset the task's internal state. Called when this task is interrupted by another one
     */
    public void func_75251_c() {
        this.targetMate = null;
        this.spawnBabyDelay = 0;
    }

    /**
     * Keep ticking a continuous task that has already been started
     */
    public void func_75246_d() {
        this.animal.func_70671_ap().func_75651_a(this.targetMate, 10.0F, (float) this.animal.func_70646_bf());
        this.animal.func_70661_as().func_75497_a(this.targetMate, this.moveSpeed);
        ++this.spawnBabyDelay;

        if (this.spawnBabyDelay >= 60 && this.animal.func_70068_e(this.targetMate) < distance) {
            this.spawnBaby();
        }
    }

    /**
     * Loops through nearby animals and finds another animal of the same type that can be mated with. Returns the first
     * valid mate found.
     */
    private EntityAnimal getNearbyMate() {
        List<EntityAnimal> list = this.world.<EntityAnimal>func_72872_a(this.mateClass, this.animal.func_174813_aQ().func_186662_g(8.0D));
        double d0 = Double.MAX_VALUE;
        EntityAnimal entityanimal = null;

        for (EntityAnimal entityanimal1 : list) {
            if (this.animal.func_70878_b(entityanimal1) && this.animal.func_70068_e(entityanimal1) < d0) {
                entityanimal = entityanimal1;
                d0 = this.animal.func_70068_e(entityanimal1);
            }
        }

        return entityanimal;
    }

    /**
     * Spawns a baby animal of the same type.
     */
    private void spawnBaby() {
        EntityAgeable entityageable = this.animal.func_90011_a(this.targetMate);

        final net.minecraftforge.event.entity.living.BabyEntitySpawnEvent event = new net.minecraftforge.event.entity.living.BabyEntitySpawnEvent(animal, targetMate, entityageable);
        final boolean cancelled = net.minecraftforge.common.MinecraftForge.EVENT_BUS.post(event);
        entityageable = event.getChild();

        if (cancelled) {
            //Reset the "inLove" state for the animals
            this.animal.func_70873_a(6000);
            this.targetMate.func_70873_a(6000);
            this.animal.func_70875_t();
            this.targetMate.func_70875_t();
            return;
        }

        if (entityageable != null) {
            EntityPlayerMP entityplayermp = this.animal.func_191993_do();

            if (entityplayermp == null && this.targetMate.func_191993_do() != null) {
                entityplayermp = this.targetMate.func_191993_do();
            }

            if (entityplayermp != null) {
                entityplayermp.func_71029_a(StatList.field_151186_x);
                CriteriaTriggers.field_192134_n.func_192168_a(entityplayermp, this.animal, this.targetMate, entityageable);
            }

            this.animal.func_70873_a(6000);
            this.targetMate.func_70873_a(6000);
            this.animal.func_70875_t();
            this.targetMate.func_70875_t();
            entityageable.func_70873_a(-24000);

            Vec3d a = event.getParentA().func_174791_d();
            Vec3d b = event.getParentB().func_174791_d();
            Vec3d m = getMidpoint(a, b);
            entityageable.func_70012_b(m.field_72450_a, m.field_72448_b, m.field_72449_c, 0.0F, 0.0F);

            this.world.func_72838_d(entityageable);
            Random random = this.animal.func_70681_au();

            for (int i = 0; i < 7; ++i) {
                double d0 = random.nextGaussian() * 0.02D;
                double d1 = random.nextGaussian() * 0.02D;
                double d2 = random.nextGaussian() * 0.02D;
                double d3 = random.nextDouble() * (double) entityageable.field_70130_N * 2.0D - (double) entityageable.field_70130_N;
                double d4 = 0.5D + random.nextDouble() * (double) entityageable.field_70131_O;
                double d5 = random.nextDouble() * (double) entityageable.field_70130_N * 2.0D - (double) entityageable.field_70130_N;
                entityageable.field_70170_p.func_175688_a(EnumParticleTypes.HEART, entityageable.field_70165_t + d3, entityageable.field_70163_u + d4, entityageable.field_70161_v + d5, d0, d1, d2);
            }


            if (entityageable.field_70170_p.func_82736_K().func_82766_b("doMobLoot")) {
                entityageable.field_70170_p.func_72838_d(new EntityXPOrb(entityageable.field_70170_p, entityageable.field_70165_t, entityageable.field_70163_u, entityageable.field_70161_v, random.nextInt(7) + 1));
            }
        }
    }
}
